/*
 * (C) Copyright 2016 Kurento (http://kurento.org/)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package org.kurento.modulecreator;

import java.util.Arrays;
import java.util.List;

import org.kurento.modulecreator.codegen.Error;

public class Result {

  private boolean success;
  private List<Error> errors;

  public Result() {
    this.success = true;
  }

  public Result(List<Error> errors) {
    this.errors = errors;
    this.success = false;
  }

  public Result(Error... errors) {
    this(Arrays.asList(errors));
  }

  public boolean isSuccess() {
    return success;
  }

  public void showErrorsInConsole() {
    System.out.println("Errors:");
    for (Error error : errors) {
      System.out.println(error);
    }
  }

  public List<Error> getErrors() {
    return errors;
  }
}
